import React, { useState } from 'react';
import { Button, DropdownContainer, DropdownItem, DropdownItemContainer, DropdownTrigger } from '../../components';
import { Tooltip, TooltipContent, TooltipProvider, TooltipTrigger } from '../../components/tooltip';
import { Add, Close, Fullscreen } from '@mui/icons-material';
import { ControlContainer } from '../../components/controls';
import { Tabs, Tab } from '../../components/tabs';
import { useAppDispatch, useVisualization } from '../../data-access';
import { addVisualization, removeVisualization, reorderVisState, setActiveVisualization } from '../../data-access/store/visualizationSlice';
import { Visualizations } from './VisualizationPanel';

export default function VisualizationTabBar(props: { fullSize: () => void }) {
  const { activeVisualizationIndex, openVisualizationArray } = useVisualization();
  const [open, setOpen] = useState(false);
  const dispatch = useAppDispatch();

  const handleDragStart = (e: React.DragEvent<HTMLDivElement>, i: number) => {
    e.dataTransfer.setData('text/plain', i.toString());
  };

  const handleDragOver = (e: React.DragEvent<HTMLDivElement>) => {
    e.preventDefault();
  };

  const handleDrop = (e: React.DragEvent<HTMLDivElement>, i: number) => {
    e.preventDefault();
    const draggedVisIndex = e.dataTransfer.getData('text/plain');
    dispatch(reorderVisState({ id: Number(draggedVisIndex), newPosition: i }));
  };

  const onSelect = async (id?: number) => {
    if (id === undefined) return;
    dispatch(setActiveVisualization(id));
  };

  const onDelete = (id: number) => {
    dispatch(removeVisualization(id));
  };

  return (
    <div className="sticky shrink-0 top-0 flex items-stretch justify-between h-7 bg-secondary-100 border-b border-secondary-200 max-w-full">
      <div className="flex items-center">
        <h1 className="text-xs font-semibold text-secondary-600 px-2 truncate">Visualization</h1>
      </div>

      <Tabs>
        {openVisualizationArray.map((vis, i) => {
          const isActive = activeVisualizationIndex === i;
          return (
            <Tab
              key={i}
              activeTab={isActive}
              text={vis.name}
              onClick={() => onSelect(i)}
              onDragStart={(e) => handleDragStart(e, i)}
              onDragOver={(e) => handleDragOver(e)}
              onDrop={(e) => handleDrop(e, i)}
              draggable
            >
              <Button
                variantType="secondary"
                variant="ghost"
                rounded
                size="2xs"
                iconComponent={<Close />}
                onClick={(e) => {
                  e.stopPropagation();
                  onDelete(i);
                }}
              />
            </Tab>
          );
        })}
      </Tabs>
      <div className="flex items-center shrink-0 px-0.5">
        <TooltipProvider delayDuration={0}>
          <Tooltip>
            <TooltipTrigger>
              <DropdownContainer open={open} onOpenChange={setOpen}>
                <DropdownTrigger onClick={() => setOpen((v) => !v)}>
                  <Button as={'a'} variantType="secondary" variant="ghost" size="xs" iconComponent={<Add />} onClick={() => {}} />
                </DropdownTrigger>
                <DropdownItemContainer>
                  {Object.keys(Visualizations).map((name, i) => (
                    <DropdownItem
                      value={name}
                      key={name}
                      className=""
                      onClick={async (e) => {
                        // onSelect(i);
                        const component = await Visualizations[name]();
                        dispatch(addVisualization({ ...component.default.settings, name: name, id: name }));
                        setOpen(false);
                      }}
                    ></DropdownItem>
                  ))}
                </DropdownItemContainer>
              </DropdownContainer>
            </TooltipTrigger>
            <TooltipContent>
              <p>Add visualization</p>
            </TooltipContent>
          </Tooltip>
        </TooltipProvider>
      </div>
      <div className="shrink-0 sticky right-0 px-0.5 ml-auto items-center flex">
        <ControlContainer>
          <TooltipProvider delayDuration={0}>
            <Tooltip>
              <TooltipTrigger asChild>
                <Button variantType="secondary" variant="ghost" size="xs" iconComponent={<Fullscreen />} onClick={props.fullSize} />
              </TooltipTrigger>
              <TooltipContent>
                <p>Full screen</p>
              </TooltipContent>
            </Tooltip>
          </TooltipProvider>
        </ControlContainer>
      </div>
    </div>
  );
}
